# -*- coding: utf-8 -*-
"""fair_pred

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1unSJuvRr3WKVGwADz2BsWBy3u-iLmyK-
"""

"""	FairPred FairPredict:
	Gets data and protected column and 
"""

import os
import pandas as pd
import numpy as np 
from typing import List
from aif360.metrics import BinaryLabelDatasetMetric
from aif360.algorithms.preprocessing.reweighing import Reweighing
from collections import defaultdict

DEFAULT = 'Testing'

class FairPredictor:
	"""
	Class used to call FairPredictor 
	"""

	def __init__(self,unprivileged_groups,privileged_groups):
		self.metric = None
		self.unprivileged_groups = unprivileged_groups
		self.privileged_groups = privileged_groups

	def fit(self, data):
		self.describe_metrics(data)
		RW = Reweighing(unprivileged_groups=self.unprivileged_groups,
		       privileged_groups=self.privileged_groups)
		RW.fit(data)
		self.metric = RW

	def transform(self, data):
		trans_data = self.metric.transform(data)	
		self.describe_metrics(trans_data)
		return trans_data


	def describe_metrics(self, dataset):
		metric_arrs = defaultdict(list)
		metric = BinaryLabelDatasetMetric(dataset, 
			unprivileged_groups=self.unprivileged_groups,
			privileged_groups=self.privileged_groups)
		metric_arrs['bal_acc'].append((metric.num_positives() + metric.num_negatives()) / 2)
		metric_arrs['disp_imp'].append(metric.disparate_impact())
		metric_arrs['stat_par_diff'].append(metric.statistical_parity_difference())
		metric_arrs['consistency'].append(metric.consistency())
		best_ind = np.argmax(metric_arrs['bal_acc'])
		print("Best balanced accuracy: {:6.4f}".format(metric_arrs['bal_acc'][best_ind]))
		#  disp_imp_at_best_ind = np.abs(1 - np.array(metrics['disp_imp']))[best_ind]
		disp_imp_at_best_ind = 1 - min(metric_arrs['disp_imp'][best_ind], 1/metric_arrs['disp_imp'][best_ind])
		print("Corresponding 1-min(DI, 1/DI) value: {:6.4f}".format(disp_imp_at_best_ind))
		print("Corresponding statistical parity difference value (mean difference): {:6.4f}"\
			.format(metric_arrs['stat_par_diff'][best_ind]))







